#!/bin/ksh
#set -x

PATCH_NAME="P510L-02D"
IDSRELEASE="5.1"
idsrelease="5\.1"

myid=$(id | cut -f 2 -d '(' | cut -f 1 -d ')')
if [[ ${myid} != "root" ]]; then
   print "You must be root to install this patch."
   return 1
fi

if ! ( rpm -qa | grep ${idsrelease} | grep ldap-client >/dev/null ); then
   print "\nThis patch requires IDS v${IDSRELEASE}"
   return 1
fi

if ( ps -ef | grep ibmdiradm | grep -v grep ) ; then
   print "ibmdiradm must be stopped before applying this patch."
   return 1
fi

if ( ps -ef | grep ibmslapd | grep -v grep ) ; then
   print "ibmslapd must be stopped before applying this patch."
   return 1
fi

PATCH_DIR=$(dirname ${0})
cd ${PATCH_DIR}
PATCH_DIR=$(pwd)

if [[ ! -a data/all_files.list ]]; then
   print "Error: can't find ${PATCH_DIR}/data/all_files.list"
   return 1
fi

if [[ ! -a data/UNDO.tar.gz ]]; then
   ( rm data/UNDO_files.* 2>&1 ) >/dev/null
   if [[ ! -a data/UNDO_files.list ]]; then
      for filename in $(cat data/all_files.list); do
         if [[ -f /usr/ldap/${filename} ]]; then
            print ${filename} >>data/UNDO_files.list
            cksum /usr/ldap/${filename} >>data/UNDO_files.sum
         fi
      done
   fi

   print "Backing up existing files..."
   cd /usr/ldap
   tar cf ${PATCH_DIR}/data/UNDO.tar $(cat ${PATCH_DIR}/data/UNDO_files.list)
   if [[ $? -ne 0 ]]; then
      print "\nUnable to backup existing files."
      print "The patch will not be installed."
      return 1
   fi
   cd ${PATCH_DIR}
   gzip data/UNDO.tar
fi

if [[ ! -a data/UNDO_files.list ]]; then
   # This should never happen, but just in case...
   print "Regenerating patch file list..."
   gzip -cd data/UNDO.tar.gz | tar tf - >data/UNDO_files.list
   if [[ $? -ne 0 ]]; then
      print "\nUnable to generate filelist."
      print "the patch will not be installed."
      return 1
   fi
fi

print "Installing patch files ..."
( rm -rf tmp 2>&1 ) >/dev/null
mkdir tmp
cd tmp

gzip -cd ../data/patch.tar.gz | tar xf - bin/${PATCH_NAME}Readme.txt bin/${PATCH_NAME}.txt $(cat ../data/UNDO_files.list)
if [[ $? -ne 0 ]]; then
   print "\nError extracting patch"
   cd ..
   rm -rf tmp
   return 1
fi

## Restore backup files if copy fails
/bin/cp -r * /usr/ldap
if [[ $? -ne 0 ]]; then
   print "The patch could not be installed."
   print "Restoring backup files ..."
   cd /usr/ldap
   ( rm -rf bin/${PATCH_NAME}* 2>&1 ) >/dev/null
   ## Prompt user any errors if restoring fails
   gzip -cd ${PATCH_DIR}/data/UNDO.tar.gz | tar xf -
   if [[ $? -ne 0 ]]; then
      print "Error restoring files."
      print "Please manually restore the backup files from ${PATCH_DIR}/data/UNDO.tar.gz"
      return 1
   fi
   return 1
fi

cd ${PATCH_DIR}
rm -rf tmp

## Remove any existing PATCH_files
( rm data/PATCH_files.* 2>&1 ) >/dev/null
## Get sum and size for Readme, signature file and all the rest
for filename in bin/${PATCH_NAME}Readme.txt bin/${PATCH_NAME}.txt $(cat data/UNDO_files.list); do
   cksum /usr/ldap/${filename} >>data/PATCH_files.sum
done

print "The patch has been successfully installed."
print "You can restart ibmdiradm and ibmslapd."
print "\n(Note: The patch can be uninstalled by running rejectPatch.sh"
print "       only if data/UNDO.tar.gz is not removed. )"

return 0
